/*
 * Exotic (EXtatic.Org Test InfrastuCture)
 * Copyright (c) 2007, Jan Vidar Krey
 */

#include "autotest.h"

Autotest::TestMachine::TestMachine(int argc, char** argv) :
	first(0),
	test(0),
	fail(0),
	pass(0),
	config_show_summary(Default),
	config_show_pass(Default),
	config_show_fail(Default)
{
	for (int n = 1; n < argc; n++)
	{
		if (!strcmp(argv[n], "-h") || !strcmp(argv[n], "--help"))
		{
			help(argv[0]);
			exit(0);
		}
		
		if (!strcmp(argv[n], "-v") || !strcmp(argv[n], "--version"))
		{
			version();
			exit(0);
		}
		
		if (!strcmp(argv[n], "-s") || !strcmp(argv[n], "--summary"))
		{
			config_show_summary = On;
			continue;
		}
		
		if (!strcmp(argv[n], "-f") || !strcmp(argv[n], "--fail"))
		{
			config_show_fail = On;
			continue;
		}
		
		if (!strcmp(argv[n], "-p") || !strcmp(argv[n], "--pass"))
		{
			config_show_pass = On;
			continue;
		}
		
		printf("Unknown argument: %s\n\n", argv[n]);
		help(argv[0]);
		exit(0);
	}
	
	if (config_show_summary == On) {
		if (config_show_pass == Default) config_show_pass = Off;
		if (config_show_fail == Default) config_show_fail = Off;
	
	} else if (config_show_pass == On) {
		if (config_show_summary == Default) config_show_summary = Off;
		if (config_show_fail == Default) config_show_fail = Off;
	} else if (config_show_fail == On) {
		if (config_show_summary == Default) config_show_summary = Off;
		if (config_show_fail == Default) config_show_fail = Off;
	} else {
		if (config_show_summary == Default) config_show_summary = On;
		if (config_show_fail == Default) config_show_fail = On;
		if (config_show_pass == Default) config_show_pass = On;
	}
	
}

Autotest::TestMachine::~TestMachine()
{
	delete first;
}

void Autotest::TestMachine::version()
{
	printf("EXtatic.Org Test InfrastruCture: exotic " VERSION "\n\n");
	printf("Copyright (C) 2007 Jan Vidar Krey, janvidar@extatic.org\n");
	printf("This is free software with ABSOLUTELY NO WARRANTY.\n\n");
}

void Autotest::TestMachine::help(const char* program)
{
	printf("Usage: %s [OPTIONS]\n\n", program);
	printf("Options:\n");
	printf("  --help      -h    Show this message\n");
	printf("  --version   -v    Show version\n");
	printf("  --summary   -s    Only show summary\n");
	printf("  --fail      -f    Only show test failures\n");
	printf("  --pass      -p    Only show test passes\n");
	printf("\nExamples:\n");
	printf("  %s -s -f\n\n", program);
}

void Autotest::TestMachine::summary()
{
	printf("\n");
	printf("--------------------------------------------\n");
	printf("Results:\n");
	printf(" * Total tests run:  %8d\n", (int) (pass+fail));
	printf(" * Tests passed:     %8d (~%d%%)\n", (int) pass, (int) (100*pass / (pass+fail)));
	printf(" * Tests failed:     %8d (~%d%%)\n", (int) fail, (int) (100*fail / (pass+fail)));
	printf("--------------------------------------------\n");
}

int Autotest::TestMachine::run()
{
	register_tests(this);

	if (!first) {
		printf("* No tests to run\n");
		return 0;
	}
	
	TestBase* current = first;
	while (current) {
		bool ok = current->test();
		if (ok) {
			if (config_show_pass == On) printf("* PASS test '%s'\n", current->name);
			pass++;
		} else {
			if (config_show_fail == On) printf("* FAIL test '%s'\n", current->name);
			fail++;
		}
		current = current->next;
	}
	
	if (config_show_summary == On)
		summary();

	return (fail) ? -fail : pass;
}

Autotest::TestBase::TestBase(const char* name_)
{
	name = name_ ? strdup(name_) : 0;
	next = 0;
}

Autotest::TestBase::~TestBase()
{
	delete next;
	free(name);
}

bool Autotest::TestBase::test() {
	return false;
}

/*
int main(int argc, char** argv)
{
	Autotest::TestMachine machine(argc, argv);
	machine.run();
	
}
*/

