/*
 * Exotic (EXtatic.Org Test InfrastuCture)
 * Copyright (c) 2007, Jan Vidar Krey
 */
#ifndef HAVE_EXOTIC_AUTOTEST_H
#define HAVE_EXOTIC_AUTOTEST_H

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

namespace Autotest {

class TestMachine;

extern "C++" void register_tests(Autotest::TestMachine*);

class TestBase {
	public:
		TestBase(const char* name);
		virtual ~TestBase();
	
		/**
		 * Each test is based on this function, and it's
		 * return value determines if the test passed (true) or failed (failed).
		 */
		virtual bool test();

	public:
		TestBase* next;
		char* name;
};

class TestMachine {
	public:
		TestMachine(int argc, char** argv);
		
		virtual ~TestMachine();
		
		/**
		 * Run the test machine
		 */
		int run();

		/**
		 * Add a test to the test machine.
		 */
		void add(TestBase* current)
		{
			if (!current) return;
			if (!first) {
				first = current;
				test = current;
			} else {
				test->next = current;
				test = current;
			}
		}
	
		/**
		 * Display a summary of all the tests passed
		 * and failed.
		 */
		void summary();

		/**
		 * Display a help message for how to run
		 * the test setup.
		 */
		void help(const char*);

		/**
		 * Displays the exotic version.
		 */
		void version();
		
	protected:
		TestBase* first;
		TestBase* test;
		
	private:
		size_t fail;
		size_t pass;
		
	private:
		enum Toggle { Default, Off, On };
		enum Toggle config_show_summary;
		enum Toggle config_show_pass;
		enum Toggle config_show_fail;
};

}

#define DISABLED_QC_TEST(NAME, block)

#define QC_TEST(NAME, block) \
	class automatic_test_ ## NAME : public Autotest::TestBase { \
		public: \
			automatic_test_ ## NAME() : Autotest::TestBase(#NAME) { } \
			~automatic_test_ ## NAME() { } \
			bool test() block \
	};
	

#define QC_REG_TEST(machine, NAME) \
	machine->add(new automatic_test_ ## NAME())
	

#endif // HAVE_EXOTIC_AUTOTEST_H


